;====================================
;Project Name: Listbox Example    
;Creator: Max_Power
;Start Date: 2-5-04 
;Version: 1.0
;Compiler: MASM (Win32Asm)
;====================================

;
;Allocation Directives
;
    .486                       ; create 32 bit code
    .model flat, stdcall       ; 32 bit memory model
    option casemap :none       ; case sensitive
;
;Includes
;
    include \masm32\include\dialog.inc
    include \masm32\include\dialogs.inc
;
;Protocols
;  
    DlgProc PROTO :DWORD,:DWORD,:DWORD,:DWORD
    OpenSavedList PROTO :DWORD
    SaveListbox PROTO :DWORD
    MultiSelectSelectedItems PROTO :DWORD
;
;Constants
;
    .const
    IDC_OPEN equ 3000
    IDC_SAVE equ 3001
    IDC_CLEAR equ 3002
    IDC_SELECTED equ 3003
    IDC_LIST equ 300
    MAXSIZE equ 260
;
;Initialized Data
;
    .data
    ofn   OPENFILENAME <>
    FilterString db "All Files",0,"*.*",0
        db "Text Files",0,"*.txt",0,0
    buffer db MAXSIZE dup(0)
    CRLF db 0Dh,0Ah,00h,00h
    nWritten dd 0
    nCount dd 0
    nCurrent dd 0
;
;Uninitialized Data
;
    .data?
    hWnd dd ? 
    hList dd ?
    hFile HANDLE ?
    szItem db 100 dup (?)
;
;Macros
;
    FUNC MACRO parameters:VARARG
        invoke parameters
        EXITM <eax>
    ENDM
;
;Code
;
    .code
start:
      mov hInstance, FUNC(GetModuleHandle,NULL)
      call Main
      invoke ExitProcess,eax
Main proc
    ;Initialize Dialog
    Dialog "Listbox Example","MS Sans Serif",10, \            ; caption,font,pointsize
            WS_OVERLAPPED or WS_SYSMENU or DS_CENTER or DS_3DLOOK or DS_MODALFRAME or WS_SYSMENU or WS_CAPTION \  ; style
            or WS_MINIMIZEBOX, \    
            5, \                                            ; control count
            10,10,111,145, \                                ; x y co-ordinates
            1024                                            ; memory buffer size
    ;Initialize Constrols
    DlgButton "Open",WS_TABSTOP,9,107,43,13,IDC_OPEN
    DlgButton "Save",WS_TABSTOP,58,107,43,13,IDC_SAVE
    DlgButton "Clear",WS_TABSTOP,9,126,43,13,IDC_CLEAR
    DlgButton "Selected",WS_TABSTOP,58,126,43,13,IDC_SELECTED
    DlgList LBS_NOINTEGRALHEIGHT or LBS_HASSTRINGS or WS_VSCROLL or WS_TABSTOP or WS_BORDER,\
        14,7,82,93,IDC_LIST
    CallModalDialog hInstance,0,DlgProc,NULL
    ret
Main endp


DlgProc proc hWin:DWORD,uMsg:DWORD,wParam:DWORD,lParam:DWORD
    .IF uMsg == WM_INITDIALOG
        mov ofn.lStructSize,SIZEOF ofn
	  push hWnd
	  pop  ofn.hWndOwner
	  push hInstance
	  pop  ofn.hInstance
	  mov  ofn.lpstrFilter, OFFSET FilterString
	  mov  ofn.lpstrFile, OFFSET buffer
	  mov  ofn.nMaxFile,MAXSIZE
	  mov eax,hWin
        mov hWnd,eax
        invoke SetFocus, FUNC(GetDlgItem,hWin,IDC_OPEN)
        mov hList, FUNC(GetDlgItem,hWin,IDC_LIST)
    .ELSEIF uMsg == WM_COMMAND
        .IF wParam == IDC_OPEN
            invoke OpenSavedList,IDC_LIST
        .ELSEIF wParam == IDC_SAVE
            invoke SaveListbox,IDC_LIST
        .ELSEIF wParam == IDC_CLEAR
            invoke SendMessage,hList,LB_RESETCONTENT ,0,0
        .ELSEIF wParam == IDC_SELECTED
            invoke MultiSelectSelectedItems,IDC_LIST
        .ENDIF
    .ELSEIF uMsg == WM_CLOSE
        invoke EndDialog,hWin,0
    .ENDIF
    xor eax, eax
    ret
DlgProc endp

OpenSavedList proc uses esi edi ebx idList:DWORD
LOCAL Char[2]		:BYTE
LOCAL Line[254]	:BYTE
LOCAL BytesRead		:DWORD
    mov hList, FUNC(GetDlgItem,hWnd,idList)
    mov  ofn.Flags, OFN_FILEMUSTEXIST or \
        OFN_PATHMUSTEXIST or OFN_LONGNAMES or\
        OFN_EXPLORER or OFN_HIDEREADONLY
    invoke GetOpenFileName, ADDR ofn
    .IF eax==TRUE
        invoke CreateFile,ADDR buffer,\
            GENERIC_READ or GENERIC_WRITE ,\
            FILE_SHARE_READ or FILE_SHARE_WRITE,\
            NULL,OPEN_EXISTING,FILE_ATTRIBUTE_ARCHIVE,\
            NULL
        mov hFile,eax
	  lea esi,Char
	  lea edi,Line
	  xor ebx,ebx
ReadLine:
	  invoke ReadFile,hFile,esi,1,ADDR BytesRead,NULL
	  cmp BytesRead,NULL
	  je @F
	  mov al,[esi]
	  cmp al,0Dh
	  je @F
	  mov [edi],al
	  inc edi
	  inc ebx
	  jmp ReadLine
@@:
	  mov [edi],BYTE PTR 0
	  lea edi,Line
	  invoke SetFilePointer,hFile,1,NULL,FILE_CURRENT
        invoke SendMessage,hList,LB_ADDSTRING,0,ADDR Line
	  cmp BytesRead,NULL
	  je @F
	  xor ebx,ebx
	  jmp ReadLine
@@:
    .ENDIF
    ret
OpenSavedList endp

SaveListbox proc idList:DWORD
    mov hList, FUNC(GetDlgItem,hWnd,idList)
    mov nCurrent,0
    mov  ofn.Flags, OFN_FILEMUSTEXIST or \
        OFN_PATHMUSTEXIST or OFN_LONGNAMES or\
        OFN_EXPLORER or OFN_HIDEREADONLY
    invoke GetSaveFileName, ADDR ofn
    ;Create file
    mov hFile, FUNC(CreateFile,ADDR buffer,GENERIC_WRITE,FILE_SHARE_WRITE,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL)
    ;Get number of items in listbox
    mov nCount, FUNC(SendMessage,hList,LB_GETCOUNT,0,0)
    ;Loop threw getting whatever the next item on the list is and writing it to a text file
    .REPEAT
        invoke SendMessage,hList,LB_GETTEXT,nCurrent,ADDR szItem
        invoke lstrcat,ADDR szItem,ADDR CRLF
        invoke lstrlen,ADDR szItem
        mov edx,eax
        invoke WriteFile,hFile,ADDR szItem,edx,ADDR nWritten,NULL
        inc nCurrent
        mov eax,nCount
    .UNTIL nCurrent == eax
    invoke CloseHandle, hFile
    ret
SaveListbox endp

MultiSelectSelectedItems proc idList:DWORD
    mov hList, FUNC(GetDlgItem,hWnd,idList)
    mov nCurrent, FUNC(SendMessage,hList,LB_GETCURSEL,0,0)
    invoke SendMessage,hList,LB_GETTEXT,nCurrent,ADDR szItem
    invoke MessageBox,hWnd,ADDR szItem,NULL,MB_OK
    ret
MultiSelectSelectedItems endp

end start
